@@ -1,24 +1,26 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2020 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
+#
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author:
-#  Arseniy Sharoglazov <mohemiv@gmail.com> / Positive Technologies (https://www.ptsecurity.com/)
-#
 # Description:
-#  A tool for connecting to MS Exchange via RPC over HTTP v2
+#   A tool for connecting to MS Exchange via RPC over HTTP v2
 #
-# Notes about -rpc-hostname:
-#  Our RPC over HTTP v2 implementation tries to extract the
-#  target's NetBIOS name via NTLMSSP and use it as RPC Server name.
-#  If it fails, you have to manually get the target RPC Server name
-#  from the Autodiscover service and set it in the -rpc-hostname parameter.
+#   Notes about -rpc-hostname:
+#     Our RPC over HTTP v2 implementation tries to extract the
+#     target's NetBIOS name via NTLMSSP and use it as RPC Server name.
+#     If it fails, you have to manually get the target RPC Server name
+#     from the Autodiscover service and set it in the -rpc-hostname parameter.
+#
+# Author:
+#   Arseniy Sharoglazov <mohemiv@gmail.com> / Positive Technologies (https://www.ptsecurity.com/)
 #
 # References:
-#  https://swarm.ptsecurity.com/attacking-ms-exchange-web-interfaces/
+#   - https://swarm.ptsecurity.com/attacking-ms-exchange-web-interfaces/
 #
 
 from __future__ import print_function
@@ -33,6 +35,7 @@ from six import PY3
 from impacket import uuid, version
 from impacket.http import AUTH_BASIC
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket.structure import parse_bitmask
 from impacket.dcerpc.v5 import transport, nspi
 from impacket.mapi_constants import PR_CONTAINER_FLAGS_VALUES, MAPI_PROPERTIES
@@ -966,13 +969,7 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
-    domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?([^:]*)').match(options.target).groups('')
-
-    #In case the password contains '@'
-    if '@' in remoteName:
-        password = password + '@' + remoteName.rpartition('@')[0]
-        remoteName = remoteName.rpartition('@')[2]
+    domain, username, password, remoteName = parse_target(options.target)
 
     if domain is None:
         domain = ''
