@@ -1,22 +1,20 @@
-#!/usr/bin/python2
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/bin/python
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2018 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Karma SMB
-#
-# Author:
-#  Alberto Solino (@agsolino) 
-#  Original idea by @mubix
-#
 # Description:
+#   Karma SMB
+#
 #   The idea of this script is to answer any file read request
-#   with a set of predefined contents based on the extension 
+#   with a set of predefined contents based on the extension
 #   asked, regardless of the sharename and/or path.
-#   When executing this script w/o a config file the pathname 
-#   file contents will be sent for every request. 
+#   When executing this script w/o a config file the pathname
+#   file contents will be sent for every request.
 #   If a config file is specified, format should be this way:
 #      <extension> = <pathname>
 #   for example:
@@ -27,10 +25,10 @@
 #   The SMB2 support works with a caveat. If two different
 #   filenames at the same share are requested, the first
 #   one will work and the second one will not work if the request
-#   is performed right away. This seems related to the 
+#   is performed right away. This seems related to the
 #   QUERY_DIRECTORY request, where we return the files available.
 #   In the first try, we return the file that was asked to open.
-#   In the second try, the client will NOT ask for another 
+#   In the second try, the client will NOT ask for another
 #   QUERY_DIRECTORY but will use the cached one. This time the new file
 #   is not there, so the client assumes it doesn't exist.
 #   After a few seconds, looks like the client cache is cleared and
@@ -39,23 +37,31 @@
 #
 #   SMB1 seems to be working fine on that scenario.
 #
-#   ToDo:
-#   [ ] A lot of testing needed under different OSes. 
+# Author:
+#   Alberto Solino (@agsolino)
+#   Original idea by @mubix
+#
+# ToDo:
+#   [ ] A lot of testing needed under different OSes.
 #       I'm still not sure how reliable this approach is.
 #   [ ] Add support for other SMB read commands. Right now just
 #       covering SMB_COM_NT_CREATE_ANDX
-#   [ ] Disable write request, now if the client tries to copy 
-#       a file back to us, it will overwrite the files we're 
+#   [ ] Disable write request, now if the client tries to copy
+#       a file back to us, it will overwrite the files we're
 #       hosting. *CAREFUL!!!*
 #
 
-
+from __future__ import division
+from __future__ import print_function
 import sys
 import os
 import argparse
 import logging
 import ntpath
-import ConfigParser
+try:
+    import ConfigParser
+except ImportError:
+    import configparser as ConfigParser
 from threading import Thread
 
 from impacket.examples import logger
@@ -144,9 +150,9 @@ class KarmaSMBServer(Thread):
     def findFirst2(self, connId, smbServer, recvPacket, parameters, data, maxDataCount):
         connData = smbServer.getConnectionData(connId)
 
-        respSetup = ''
-        respParameters = ''
-        respData = ''
+        respSetup = b''
+        respParameters = b''
+        respData = b''
         findFirst2Parameters = smb.SMBFindFirst2_Parameters( recvPacket['Flags2'], data = parameters)
 
         # 1. Let's grab the extension and map the file's contents we will deliver
@@ -156,12 +162,12 @@ class KarmaSMBServer(Thread):
         _, origPathNameExtension = os.path.splitext(origPathName)
         origPathNameExtension = origPathNameExtension.upper()[1:]
 
-        if self.extensions.has_key(origPathNameExtension.upper()):
+        if origPathNameExtension.upper() in self.extensions:
             targetFile = self.extensions[origPathNameExtension.upper()]
         else:
             targetFile = self.defaultFile
 
-        if connData['ConnectedShares'].has_key(recvPacket['Tid']):
+        if recvPacket['Tid'] in connData['ConnectedShares']:
             path = connData['ConnectedShares'][recvPacket['Tid']]['path']
 
             # 2. We call the normal findFirst2 call, but with our targetFile
@@ -192,7 +198,7 @@ class KarmaSMBServer(Thread):
                     if len(connData['SIDs']) == 0:
                        sid = 1
                     else:
-                       sid = connData['SIDs'].keys()[-1] + 1
+                       sid = list(connData['SIDs'].keys())[-1] + 1
                     # Store the remaining search results in the ConnData SID
                     connData['SIDs'][sid] = searchResult[i[0]:]
                     respParameters['LastNameOffset'] = totalData
@@ -253,14 +259,14 @@ class KarmaSMBServer(Thread):
         _, origPathNameExtension = os.path.splitext(origPathName)
         origPathNameExtension = origPathNameExtension.upper()[1:]
 
-        if self.extensions.has_key(origPathNameExtension.upper()):
+        if origPathNameExtension.upper() in self.extensions:
             targetFile = self.extensions[origPathNameExtension.upper()]
         else:
             targetFile = self.defaultFile
         
         # 2. We change the filename in the request for our targetFile
         ntCreateAndXData['FileName'] = encodeSMBString( flags = recvPacket['Flags2'], text = targetFile)
-        SMBCommand['Data'] = str(ntCreateAndXData)
+        SMBCommand['Data'] = ntCreateAndXData.getData()
         smbServer.log("%s is asking for %s. Delivering %s" % (connData['ClientIP'], origPathName,targetFile),logging.INFO)
 
         # 3. We call the original call with our modified data
@@ -273,31 +279,31 @@ class KarmaSMBServer(Thread):
         
         connData = smbServer.getConnectionData(connId)
 
-        respSetup = ''
-        respParameters = ''
-        respData = ''
+        respSetup = b''
+        respParameters = b''
+        respData = b''
         errorCode = 0
 
         queryPathInfoParameters = smb.SMBQueryPathInformation_Parameters(flags = recvPacket['Flags2'], data = parameters)
 
-        if connData['ConnectedShares'].has_key(recvPacket['Tid']):
+        if recvPacket['Tid'] in connData['ConnectedShares']:
             path = ''
             try:
                origPathName = decodeSMBString(recvPacket['Flags2'], queryPathInfoParameters['FileName'])
                origPathName = os.path.normpath(origPathName.replace('\\','/'))
 
-               if connData.has_key('MS15011') is False:
+               if ('MS15011' in connData) is False:
                    connData['MS15011'] = {}
 
                smbServer.log("Client is asking for QueryPathInformation for: %s" % origPathName,logging.INFO)
-               if connData['MS15011'].has_key(origPathName) or origPathName == '.':
+               if origPathName in connData['MS15011'] or origPathName == '.':
                    # We already processed this entry, now it's asking for a directory
                    infoRecord, errorCode = queryPathInformation(path, '/', queryPathInfoParameters['InformationLevel'])
                else:
                    # First time asked, asking for the file
                    infoRecord, errorCode = queryPathInformation(path, self.defaultFile, queryPathInfoParameters['InformationLevel'])
                    connData['MS15011'][os.path.dirname(origPathName)] = infoRecord
-            except Exception, e:
+            except Exception as e:
                #import traceback
                #traceback.print_exc()
                smbServer.log("queryPathInformation: %s" % e,logging.ERROR)
@@ -363,7 +369,7 @@ class KarmaSMBServer(Thread):
 
         # Are we being asked for a directory?
         if (createOptions & smb2.FILE_DIRECTORY_FILE) == 0:
-            if self.extensions.has_key(origPathNameExtension.upper()):
+            if origPathNameExtension.upper() in self.extensions:
                 targetFile = self.extensions[origPathNameExtension.upper()]
             else:
                 targetFile = self.defaultFile
@@ -379,7 +385,7 @@ class KarmaSMBServer(Thread):
             import sys
             ntCreateRequest['Buffer'] = targetFile.decode(sys.getfilesystemencoding()).encode('utf-16le')
         ntCreateRequest['NameLength'] = len(targetFile)*2
-        recvPacket['Data'] = str(ntCreateRequest)
+        recvPacket['Data'] = ntCreateRequest.getData()
 
         # 3. We call the original call with our modified data
         return self.origsmb2Create(connId, smbServer, recvPacket)
@@ -393,7 +399,7 @@ class KarmaSMBServer(Thread):
         #queryDirectoryRequest   = smb2.SMB2QueryDirectory(recvPacket['Data'])
 
         errorCode = 0xff
-        respSMBCommand['Buffer'] = '\x00' 
+        respSMBCommand['Buffer'] = b'\x00'
 
         errorCode = STATUS_SUCCESS
 
@@ -419,7 +425,7 @@ class KarmaSMBServer(Thread):
             infoRecord['LastAccessTime']    = getFileTime(atime)
             infoRecord['LastWriteTime']     = getFileTime(mtime)
             infoRecord['LastChangeTime']    = getFileTime(mtime)
-            infoRecord['ShortName']         = '\x00'*24
+            infoRecord['ShortName']         = b'\x00'*24
             #infoRecord['FileName']          = os.path.basename(origName).encode('utf-16le')
             infoRecord['FileName']          = origName.encode('utf-16le')
             padLen = (8-(len(infoRecord) % 8)) % 8
@@ -427,7 +433,7 @@ class KarmaSMBServer(Thread):
 
             respSMBCommand['OutputBufferOffset'] = 0x48
             respSMBCommand['OutputBufferLength'] = len(infoRecord.getData())
-            respSMBCommand['Buffer'] = infoRecord.getData() + '\xaa'*padLen
+            respSMBCommand['Buffer'] = infoRecord.getData() + b'\xaa'*padLen
             connData['MS15011']['FindDone'] = True
 
         smbServer.setConnectionData(connId, connData)
@@ -453,7 +459,7 @@ class KarmaSMBServer(Thread):
         errorCode = STATUS_SUCCESS
 
         ## Process here the request, does the share exist?
-        path = str(recvPacket)[treeConnectRequest['PathOffset']:][:treeConnectRequest['PathLength']]
+        path = recvPacket.getData()[treeConnectRequest['PathOffset']:][:treeConnectRequest['PathLength']]
         UNCOrShare = path.decode('utf-16le')
 
         # Is this a UNC?
@@ -473,7 +479,7 @@ class KarmaSMBServer(Thread):
             if len(connData['ConnectedShares']) == 0:
                tid = 1
             else:
-               tid = connData['ConnectedShares'].keys()[-1] + 1
+               tid = list(connData['ConnectedShares'].keys())[-1] + 1
             connData['ConnectedShares'][tid] = share
             connData['ConnectedShares'][tid]['path'] = '/'
             connData['ConnectedShares'][tid]['shareName'] = path
@@ -544,7 +550,7 @@ class KarmaSMBServer(Thread):
         if len(connData['ConnectedShares']) == 0:
            tid = 1
         else:
-           tid = connData['ConnectedShares'].keys()[-1] + 1
+           tid = list(connData['ConnectedShares'].keys())[-1] + 1
         connData['ConnectedShares'][tid] = share
         connData['ConnectedShares'][tid]['path'] = '/'
         connData['ConnectedShares'][tid]['shareName'] = path
@@ -558,7 +564,7 @@ class KarmaSMBServer(Thread):
         else:
             respData['Service']               = path
         respData['PadLen']                = 0
-        respData['NativeFileSystem']      = encodeSMBString(recvPacket['Flags2'], 'NTFS' )
+        respData['NativeFileSystem']      = encodeSMBString(recvPacket['Flags2'], 'NTFS' ).decode()
 
         respSMBCommand['Parameters']             = respParameters
         respSMBCommand['Data']                   = respData 
@@ -590,7 +596,7 @@ class KarmaSMBServer(Thread):
 if __name__ == '__main__':
     # Init the example's logger theme
     logger.init()
-    print version.BANNER
+    print(version.BANNER)
     parser = argparse.ArgumentParser(add_help = False, description = "For every file request received, this module will "
                                                                      "return the pathname contents")
     parser.add_argument("--help", action="help", help='show this help message and exit')
@@ -607,7 +613,7 @@ if __name__ == '__main__':
 
     try:
        options = parser.parse_args()
-    except Exception, e:
+    except Exception as e:
        logging.critical(str(e))
        sys.exit(1)
 
@@ -626,4 +632,3 @@ if __name__ == '__main__':
             sys.exit(1)
         else:
             pass
-
